package net.fly19.chaodict;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Random;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteException;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.AutoCompleteTextView;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TextView;

public class ChaoDictActivity extends Activity implements OnClickListener,TextWatcher,OnItemClickListener{
	
	private final String ERR_WORD = "##ûҵ##";
	SQLiteDatabase database;
	Button btnWord;
	AutoCompleteTextView actvWord;
	TextView txtvWord;
	TextView txtvWordTest;
	ListView lstvWordTest;
	int correctWord;
	String testWord;
	String[] CHNs;
	ArrayAdapter<String> AA;
	
	boolean moreMenu = false;
	
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
    	Log.d("OnCreate", "start");
    	try{
    		super.onCreate(savedInstanceState);
    		doCha();
            Log.d("OnCreate", "open db");
            database = openDatabase();
            if(database == null){
            	Log.d("OnCreate", "db error");
            	this.finish();
            }
            
            Log.d("OnCreate", "clean correctWord");
            correctWord = 0;
            testWord = new String("");
    	}catch(SecurityException e){
    		Log.d("OnCreate", "SecurityException");
    		this.finish();
    	}
        
    }
    
    public boolean doCha(){
    	Log.d("doCha", "set main view");
    	setContentView(R.layout.main);
    	Log.d("doCha", "find view");
        btnWord = (Button)findViewById(R.id.wordbutton);
        actvWord = (AutoCompleteTextView)findViewById(R.id.wordtext);
        txtvWord = (TextView)findViewById(R.id.wordvalue);

        Log.d("doCha", "add linstener");
        btnWord.setOnClickListener(this);
        actvWord.addTextChangedListener(this);
    	return true;
    }
    
    public void doShowTest(){
    	String sql = "SELECT * FROM word WHERE note_time IS NOT NULL ORDER BY check_poss DESC LIMIT 4";
    	Cursor cur = database.rawQuery(sql, null);
    	CHNs = new String[4];
    	Random rand = new Random();
    	int count = cur.getCount();
    	if(count > 1){
    		cur.moveToFirst();
    		txtvWordTest.setText(Cursor2TestString(cur));
    		int i = rand.nextInt(4);
    		CHNs[i] = "(" + (i+1) + ") " + cur.getString(cur.getColumnIndex("chn"));
    		correctWord = i;
    		Log.d("doShowTest", "correct = " + i);
    		Log.d("doShowTest", "correct chn = " + CHNs[i]);
    		while(cur.moveToNext()){
        		i = i + 1;
    			if (i > 3){
    				i = i - 4;
    			}
    			CHNs[i] = "(" + (i+1) + ") " + cur.getString(cur.getColumnIndex("chn"));
    		}
    	}else{
    		Log.d("doShowTest", "not enough word");
    	}
    	cur.close();
    }
    
    public boolean doBei(){
    	setContentView(R.layout.bei);
    	txtvWordTest = (TextView)findViewById(R.id.testtext);
        lstvWordTest = (ListView)findViewById(R.id.testlist);
        lstvWordTest.setOnItemClickListener(this);
    	doShowTest();
    	AA = new ArrayAdapter<String>(this,R.layout.list, CHNs);
    	lstvWordTest.setAdapter(AA);
    	AA.notifyDataSetChanged();
    	return true;
    }
    
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.menu, menu);
        return true;
    }
    
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
    	  switch (item.getItemId()) {
    	  case R.id.bei:
    		  doBei();
    		  return false;
    	  case R.id.cha:
    		  doCha();
    		  return false;
    	  default:
    		  return super.onOptionsItemSelected(item);
    	  }
    	}
    
    @Override
    public void onDestroy(){
    	super.onDestroy();
    	database.close();
    }
    
    private SQLiteDatabase openDatabase() {
    	String DB_PATH = android.os.Environment.getExternalStorageDirectory().getAbsolutePath();
    	String DB_FILENAME = "dict.sqlite";
    	String ERR_DB = "ݿزݿⱣSDĿ¼Уȡ" + DB_FILENAME;
    	try{
    		String DB_LONG_NAME = DB_PATH + "/" + DB_FILENAME;
    		if(!(new File(DB_LONG_NAME)).exists()){
    			//˳
    			return null;
    		}else{
    			SQLiteDatabase db = SQLiteDatabase.openDatabase(DB_LONG_NAME,null,SQLiteDatabase.OPEN_READWRITE | SQLiteDatabase.NO_LOCALIZED_COLLATORS);
    			return db;
    		}
    	}catch(SQLiteException e){
    		Log.d("open database", ERR_DB);
    		return null;
    	}
    }
    
	@Override
	public void afterTextChanged(Editable arg0) {
		if (moreMenu){
			moreMenu = false;
			btnWord.setText("   ?   ");
		}
	}

	@Override
	public void beforeTextChanged(CharSequence arg0, int arg1, int arg2,
			int arg3) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void onTextChanged(CharSequence arg0, int arg1, int arg2, int arg3) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void onClick(View v) {
		if (moreMenu){
			
		}
		String word = actvWord.getText().toString();
		String sql = "SELECT * FROM word WHERE eng LIKE ? LIMIT 1";
		Cursor cur = database.rawQuery(sql, new String[]{word.replace("\'", "\'\'")});
		String result = "";
		if(cur.getCount() > 0){
			cur.moveToFirst();
			result = Cursor2String(cur,true);
			int note_time = cur.getInt(cur.getColumnIndex("note_time"));
			if(note_time > 0){
				updateTouchTime(word);
			}else{
				updateNoteTime(word);
			}
			moreMenu = true;
			btnWord.setText("   >   ");
			txtvWord.setText(result);
			actvWord.selectAll();
		}else{
			String suggest = "";
			if (word.length()>2){
				/*
				String sql = "SELECT * FROM word WHERE eng = ? LIMIT 1";
				Cursor cur = database.rawQuery(sql, new String[]{txt});
				if(cur.getCount() > 0){
					cur.moveToFirst();
					result = Cursor2String(cur,false) + "\n";
				}
				cur.close();
				*/
				sql = "SELECT eng FROM word WHERE eng LIKE \'" + word.replace("\'", "\'\'") + "%\' LIMIT 10";
				cur = database.rawQuery(sql,null);
				if(cur.getCount() > 0){
					suggest = "---SUGGEST---";
					cur.moveToFirst();
					boolean i = true;
					while (i){
						suggest = suggest + "\n" + cur.getString(0);
						i = cur.moveToNext();
					}
				}else{
					result = ERR_WORD;
				}
				cur.close();
			}
			txtvWord.setText(result + suggest);
		}
		cur.close();
	}
	
	private String Cursor2TestString(Cursor cur){
		testWord = cur.getString(cur.getColumnIndex("eng"));
		String r = "<< " + testWord + " >>";
		String snts = cur.getString(cur.getColumnIndex("snts"));
		if (snts.length()>0){
			r = r + "\n䣺";
			snts = snts.replace("|", "\n    ");
			r = r + "\n    " + snts;
		}
		return r;
	}
	
	private String Cursor2String(Cursor cur,boolean showall){
		String result;
		result = cur.getString(cur.getColumnIndex("eng")) + ":\n" + cur.getString(cur.getColumnIndex("chn")) + "\n[" + cur.getString(cur.getColumnIndex("phon")) + "]";
		if(showall){
			result = result + "\n===================\n";
			long note_time = cur.getInt(cur.getColumnIndex("note_time"));
			if (note_time <= 0){
				result = result + "ûб\n";
				return result;
			}
			if (note_time > 0){
				result = result + "ʱ䣺" + (new Date(note_time*1000)).toLocaleString() + "\n";
			}
			long mem_count = cur.getInt(cur.getColumnIndex("mem_count"));
			result = result + "" + mem_count + "\n";
			if (mem_count > 0){
				long mem_time = cur.getInt(cur.getColumnIndex("mem_time"));
				if (mem_time > 0){
					result = result + "ϴμʱ䣺" + (new Date(mem_time*1000)).toLocaleString() + "\n";
				}
				long fail_count = cur.getInt(cur.getColumnIndex("fail_count"));
				result = result + "ʧܴ" + fail_count + "\n";
				if (fail_count > 0){
					long fail_time = cur.getInt(cur.getColumnIndex("fail_time"));
					if (fail_count > 0 && fail_time > 0){
						result = result + "ϴʧʱ䣺" + (new Date(fail_time*1000)).toLocaleString() + "\n";
					}
				}
				long succ_time = cur.getInt(cur.getColumnIndex("succ_time"));
				if (succ_time > 0){
					result = result + "ϴγɹʱ䣺" + (new Date(succ_time*1000)).toLocaleString() + "\n";
				}
			}
			float check_poss = cur.getFloat(cur.getColumnIndex("check_poss"));
			result = result + "ʣ" + check_poss + "\n";
			long touch_time = cur.getInt(cur.getColumnIndex("touch_time"));
			if (touch_time > 0){
				result = result + "鿴ʱ䣺" + (new Date(touch_time*1000)).toLocaleString() + "\n";
			}
			int sync_needed = cur.getInt(cur.getColumnIndex("sync_needed"));
			if (sync_needed > 0){
				result = result + "ͬδͬ\n";
			}else{
				result = result + "ͬͬ\n";
			}
		}
		return result;
	}
	
	private void updateNoteTime(String word){
		database.beginTransaction();
		try{
			String sql = "UPDATE word SET note_time = strftime('%s', 'now'), sync_needed = 1, check_poss = 1.0 WHERE eng = \'"+word+"\'";
			database.execSQL(sql);
			database.setTransactionSuccessful();
		}finally{
			database.endTransaction();
		}
	}
	
	private void updateTouchTime(String word){
		database.beginTransaction();
		try{
			String sql = "UPDATE word SET touch_time = strftime('%s', 'now'), sync_needed = 1 WHERE eng = \'"+word.replace("\'", "\'\'")+"\'";
			database.execSQL(sql);
			database.setTransactionSuccessful();
		}finally{
			database.endTransaction();
		}
	}

	private void updateMemTime(String word){
		updateTouchTime(word);
		database.beginTransaction();
		try{
			String sql = "UPDATE word SET mem_count = mem_count + 1,mem_time = strftime('%s', 'now') WHERE eng = \'"+word.replace("\'", "\'\'")+"\'";
			database.execSQL(sql);
			database.setTransactionSuccessful();
		}finally{
			database.endTransaction();
		}
	}
	private void updateSucc(String word){
		updateMemTime(word);
		database.beginTransaction();
		try{
			String sql = "UPDATE word SET succ_time = strftime('%s', 'now'), check_poss = check_poss * 0.5 WHERE eng = \'"+word.replace("\'", "\'\'")+"\'";
			database.execSQL(sql);
			database.setTransactionSuccessful();
		}finally{
			database.endTransaction();
		}
	}
	
	private void updateFail(String word){
		updateMemTime(word);
		database.beginTransaction();
		try{
			String sql = "UPDATE word SET fail_count = fail_count + 1, fail_time = strftime('%s', 'now'), check_poss = check_poss * 1.23 WHERE eng = \'"+word.replace("\'", "\'\'")+"\'";
			database.execSQL(sql);
			database.setTransactionSuccessful();
		}finally{
			database.endTransaction();
		}
	}
	
	private void showAlertForFail(String word){
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setMessage("\n<<" + word + ">> :\n" + CHNs[correctWord])
		       .setCancelable(false)
		       .setPositiveButton("", new DialogInterface.OnClickListener() {
		           public void onClick(DialogInterface dialog, int id){
		        	   dialog.cancel();
		           }
		       });
		AlertDialog alert = builder.create();
		alert.show();
	}
	
	@Override
	public void onItemClick(AdapterView<?> arg0, View arg1, int arg2, long arg3) {
		Log.d("onItemClick", "Click " + arg2);
		if (arg2 == correctWord){
			updateSucc(testWord);
		}else{
			showAlertForFail(testWord);
			updateFail(testWord);
		}
		doShowTest();
		AA = new ArrayAdapter<String>(this,R.layout.list, CHNs);
    	lstvWordTest.setAdapter(AA);
    	//AA.notifyDataSetChanged();
	}
}